@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2015 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2019 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -21,7 +21,7 @@ import java.sql.SQLException;
  * for the class to be pooled, which must implement {@link PooledObject}.
  * @author Thomas Hallgren
  */
-public interface ObjectPool
+public interface ObjectPool<T extends PooledObject>
 {
 	/**
 	 * Obtain a pooled object, calling its {@link PooledObject#activate()}
@@ -30,7 +30,7 @@ public interface ObjectPool
 	 * 
 	 * @return A new object or an object found in the pool.
 	 */
-	PooledObject activateInstance()
+	T activateInstance()
 	throws SQLException;
 
 	/**
@@ -38,13 +38,13 @@ public interface ObjectPool
 	 * to the pool.
 	 * @param instance The instance to passivate.
 	 */
-	void passivateInstance(PooledObject instance)
+	void passivateInstance(T instance)
 	throws SQLException;
 
 	/**
 	 * Call the {@link PooledObject#remove()} method and evict the object
 	 * from the pool.
 	 */
-	void removeInstance(PooledObject instance)
+	void removeInstance(T instance)
 	throws SQLException;
 }
