@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2019 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2020 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -16,9 +16,9 @@ package org.postgresql.pljava.jdbc;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.Reader;
-import java.io.UnsupportedEncodingException;
 import java.math.BigDecimal;
 import java.net.URL;
+import static java.nio.charset.StandardCharsets.US_ASCII;
 import java.sql.Array;
 import java.sql.Blob;
 import java.sql.Clob;
@@ -43,7 +43,7 @@ import org.postgresql.pljava.internal.ExecutionPlan;
 import org.postgresql.pljava.internal.Oid;
 
 /**
- *
+ * Implementation of {@link PreparedStatement} for the SPI connection.
  * @author Thomas Hallgren
  */
 public class SPIPreparedStatement extends SPIStatement implements PreparedStatement
@@ -181,22 +181,12 @@ public class SPIPreparedStatement extends SPIStatement implements PreparedStatem
 	@Override
 	public void setAsciiStream(int columnIndex, InputStream value, int length) throws SQLException
 	{
-		try
-		{
-			setObject(columnIndex,
-				new ClobValue(new InputStreamReader(value, "US-ASCII"), length),
-				Types.CLOB);
-		}
-		catch(UnsupportedEncodingException e)
-		{
-			throw new SQLException("US-ASCII encoding is not supported by this JVM");
-		}
+		setObject(columnIndex,
+			new ClobValue(new InputStreamReader(value, US_ASCII), length),
+			Types.CLOB);
 	}
 
-	/**
-	 * @deprecated
-	 */
-	@Override
+	@SuppressWarnings("deprecation") @Override
 	public void setUnicodeStream(int columnIndex, InputStream value, int arg2) throws SQLException
 	{
 		throw new UnsupportedFeatureException("PreparedStatement.setUnicodeStream");
