@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2019 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2021 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -13,28 +13,47 @@
 package org.postgresql.pljava.sqlj;
 
 import java.io.IOException;
-import java.lang.reflect.InvocationTargetException;
-import java.lang.reflect.Method;
-import java.lang.reflect.UndeclaredThrowableException;
+
+import java.lang.invoke.MethodHandle;
+import java.lang.invoke.MethodHandles;
+import java.lang.invoke.MethodType;
+import static java.lang.invoke.MethodType.methodType;
+
 import java.net.MalformedURLException;
 import java.net.URL;
+
+import java.security.CodeSigner;
+import java.security.CodeSource;
+import java.security.Principal;
+import java.security.ProtectionDomain;
+
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLData;
 import java.sql.SQLException;
 import java.sql.Statement;
+
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.NoSuchElementException;
+
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
+import static java.util.stream.Collectors.groupingBy;
+
+import org.postgresql.pljava.sqlgen.Lexicals.Identifier;
+
 import org.postgresql.pljava.internal.Backend;
+import org.postgresql.pljava.internal.Checked;
 import org.postgresql.pljava.internal.Oid;
-import org.postgresql.pljava.jdbc.SQLUtils;
+import static org.postgresql.pljava.internal.Privilege.doPrivileged;
+import static org.postgresql.pljava.internal.UncheckedException.unchecked;
+
+import static org.postgresql.pljava.jdbc.SQLUtils.getDefaultConnection;
 
 /*
  * Import an interface (internal-only, at least for now) that allows overriding
@@ -64,13 +83,31 @@ import org.postgresql.pljava.jdbc.SQLUtils;
 import org.postgresql.pljava.jdbc.SPIReadOnlyControl;
 
 /**
+ * Class loader to load from jars installed in the database with
+ * {@code SQLJ.INSTALL_JAR}.
  * @author Thomas Hallgren
  */
 public class Loader extends ClassLoader
 {
-	private final static Logger s_logger = Logger.getLogger(Loader.class.getName());
+	private static final Logger s_logger =
+		Logger.getLogger(Loader.class.getName());
 
-	static class EntryEnumeration implements Enumeration
+	/**
+	 * A distinguished singleton instance to serve as a type-safe "sentinel"
+	 * reference in context classloader management (as Java considers null to be
+	 * a meaningful {@code setContextClassLoader} argument).
+	 */
+	public static final ClassLoader SENTINEL = new Loader();
+
+	/**
+	 * The enumeration of URLs returned by {@code findResources}.
+	 *<p>
+	 * The returned URLs have a "dbf:" scheme and expose the integer surrogate
+	 * keys of jar entries, not a very stable way to refer to an entry in a jar,
+	 * but perhaps adequate for now, as no one will be constructing such URLs
+	 * or obtaining them except from {@code findResources} here.
+	 */
+	static class EntryEnumeration implements Enumeration<URL>
 	{
 		private final int[] m_entryIds;
 		private int m_top = 0;
@@ -85,7 +122,7 @@ public class Loader extends ClassLoader
 			return (m_top < m_entryIds.length);
 		}
 		
-		public Object nextElement()
+		public URL nextElement()
 		throws NoSuchElementException
 		{
 			if (m_top >= m_entryIds.length)
@@ -93,13 +130,15 @@ public class Loader extends ClassLoader
 			return entryURL(m_entryIds[m_top++]);
 		}
 	}
-	private static final String PUBLIC_SCHEMA = "public";
+	private static final Identifier.Simple PUBLIC_SCHEMA =
+		Identifier.Simple.fromCatalog("public");
 
-	private static final Map<String,ClassLoader>
-		s_schemaLoaders = new HashMap<String,ClassLoader>();
+	private static final Map<Identifier.Simple, ClassLoader>
+		s_schemaLoaders = new HashMap<>();
 
-	private static final Map<String,Map<Oid,Class<? extends SQLData>>>
-		s_typeMap = new HashMap<String,Map<Oid,Class<? extends SQLData>>>();
+	private static final
+		Map<Identifier.Simple, Map<Oid, Class<? extends SQLData>>>
+			s_typeMap = new HashMap<>();
 
 	/**
 	 * Removes all cached schema loaders, functions, and type maps. This
@@ -124,80 +163,84 @@ public class Loader extends ClassLoader
 	throws SQLException
 	{
 		String schema;
-		Statement stmt = SQLUtils.getDefaultConnection().createStatement();
-		ResultSet rs = null;
-		try
+		try (
+			Statement stmt = getDefaultConnection().createStatement();
+			ResultSet rs =
+				stmt.executeQuery("SELECT pg_catalog.current_schema()");
+		)
 		{
-			rs = stmt.executeQuery("SELECT pg_catalog.current_schema()");
 			if(!rs.next())
 				throw new SQLException("Unable to determine current schema");
 			schema = rs.getString(1);
 		}
-		finally
-		{
-			SQLUtils.close(rs);
-			SQLUtils.close(stmt);
-		}
-		return getSchemaLoader(schema);
+		return getSchemaLoader(Identifier.Simple.fromCatalog(schema));
 	}
 
 	/**
 	 * Obtain a loader that has been configured for the class path of the
 	 * schema named <code>schemaName</code>. Class paths are defined using the
 	 * SQL procedure <code>sqlj.set_classpath</code>.
-	 * @param schemaName The name of the schema.
+	 * @param schema The name of the schema as an Identifier.Simple.
 	 * @return A loader.
 	 */
-	public static ClassLoader getSchemaLoader(String schemaName)
+	public static ClassLoader getSchemaLoader(Identifier.Simple schema)
 	throws SQLException
 	{
-		if(schemaName == null || schemaName.length() == 0)
-			schemaName = PUBLIC_SCHEMA;
-		else
-			schemaName = schemaName.toLowerCase();
+		if(schema == null )
+			schema = PUBLIC_SCHEMA;
 
-		ClassLoader loader = s_schemaLoaders.get(schemaName);
+		ClassLoader loader = s_schemaLoaders.get(schema);
 		if(loader != null)
 			return loader;
 
-		Map classImages = new HashMap();
-		Connection conn = SQLUtils.getDefaultConnection();
-		PreparedStatement outer = null;
-		PreparedStatement inner = null;
-		try
-		{
+		/*
+		 * Under-construction map from an entry name to an array of integer
+		 * surrogate keys for entries with matching names in jars on the path.
+		 */
+		Map<String,int[]> classImages = new HashMap<>();
+
+		/*
+		 * Under-construction map from an integer entry key to a
+		 * CodeSource representing the jar it belongs to.
+		 */
+		Map<Integer,CodeSource> codeSources = new HashMap<>();
+
+		Connection conn = getDefaultConnection();
+		try (
 			// Read the entries so that the one with highest prio is read last.
 			//
-			outer = conn.prepareStatement(
-				"SELECT r.jarId" +
+			PreparedStatement outer = conn.prepareStatement(
+				"SELECT r.jarId, r.jarName" +
 				" FROM" +
 				"  sqlj.jar_repository r" +
 				"  INNER JOIN sqlj.classpath_entry c" +
 				"  ON r.jarId OPERATOR(pg_catalog.=) c.jarId" +
 				" WHERE c.schemaName OPERATOR(pg_catalog.=) ?" +
 				" ORDER BY c.ordinal DESC");
-
-			inner = conn.prepareStatement(
+			PreparedStatement inner = conn.prepareStatement(
 				"SELECT entryId, entryName FROM sqlj.jar_entry " +
 				"WHERE jarId OPERATOR(pg_catalog.=) ?");
-
+		)
+		{
 			outer.unwrap(SPIReadOnlyControl.class).clearReadOnly();
 			inner.unwrap(SPIReadOnlyControl.class).clearReadOnly();
-			outer.setString(1, schemaName);
-			ResultSet rs = outer.executeQuery();
-			try
+			outer.setString(1, schema.pgFolded());
+			try ( ResultSet rs = outer.executeQuery() )
 			{
 				while(rs.next())
 				{
+					URL jarUrl = new URL("sqlj:" + rs.getString(2));
+					CodeSource cs = new CodeSource(jarUrl, (CodeSigner[])null);
+
 					inner.setInt(1, rs.getInt(1));
-					ResultSet rs2 = inner.executeQuery();
-					try
+					try ( ResultSet rs2 = inner.executeQuery() )
 					{
 						while(rs2.next())
 						{
 							int entryId = rs2.getInt(1);
 							String entryName = rs2.getString(2);
-							int[] oldEntry = (int[])classImages.get(entryName);
+							codeSources.put(entryId, cs);
+							int[] oldEntry = classImages.get(entryName);
 							if(oldEntry == null)
 								classImages.put(entryName, new int[] { entryId });
 							else
@@ -210,22 +253,13 @@ public class Loader extends ClassLoader
 							}
 						}
 					}
-					finally
-					{
-						SQLUtils.close(rs2);
-					}
 				}
 			}
-			finally
+			catch ( MalformedURLException e )
 			{
-				SQLUtils.close(rs);
+				throw unchecked(e);
 			}
 		}
-		finally
-		{
-			SQLUtils.close(outer);
-			SQLUtils.close(inner);
-		}
 
 		ClassLoader parent = ClassLoader.getSystemClassLoader();
 		if(classImages.size() == 0)
@@ -234,11 +268,16 @@ public class Loader extends ClassLoader
 			// classpath of public schema or to the system classloader if the
 			// request already is for the public schema.
 			//
-			loader = schemaName.equals(PUBLIC_SCHEMA) ? parent : getSchemaLoader(PUBLIC_SCHEMA);
+			loader = schema.equals(PUBLIC_SCHEMA)
+				? parent : getSchemaLoader(PUBLIC_SCHEMA);
 		else
-			loader = new Loader(classImages, parent);
+		{
+			String name = "schema:" + schema.nonFolded();
+			loader = doPrivileged(() ->
+				new Loader(classImages, codeSources, parent, name));
+		}
 
-		s_schemaLoaders.put(schemaName, loader);
+		s_schemaLoaders.put(schema, loader);
 		return loader;
 	}
 
@@ -252,7 +291,7 @@ public class Loader extends ClassLoader
 	 * @return The Map, possibly empty but never <code>null</code>.
 	 */
 	public static Map<Oid,Class<? extends SQLData>> getTypeMap(
-		final String schema)
+		final Identifier.Simple schema)
 		throws SQLException
 	{
 		Map<Oid,Class<? extends SQLData>> typesForSchema =
@@ -261,34 +300,42 @@ public class Loader extends ClassLoader
 			return typesForSchema;
 
 		s_logger.finer("Creating typeMappings for schema " + schema);
-		typesForSchema = new HashMap()
+		typesForSchema = new HashMap<Oid,Class<? extends SQLData>>()
 		{
-			public Object get(Object key)
+			public Class<? extends SQLData> get(Oid key)
 			{
-				s_logger.finer("Obtaining type mapping for OID " + key + " for schema " + schema);
+				s_logger.finer("Obtaining type mapping for OID " + key +
+					" for schema " + schema);
 				return super.get(key);
 			}
 		};
 		ClassLoader loader = Loader.getSchemaLoader(schema);
-		Statement stmt = SQLUtils.getDefaultConnection().createStatement();
-		stmt.unwrap(SPIReadOnlyControl.class).clearReadOnly();
-		ResultSet rs = null;
-		try
+		try (
+			Statement stmt = Checked.Supplier.use((() ->
+			{
+				Statement s = getDefaultConnection().createStatement();
+				s.unwrap(SPIReadOnlyControl.class).clearReadOnly();
+				return s;
+			})).get();
+			ResultSet rs = stmt.executeQuery(
+				"SELECT javaName, sqlName FROM sqlj.typemap_entry");
+		)
 		{
-			rs = stmt.executeQuery("SELECT javaName, sqlName FROM sqlj.typemap_entry");
 			while(rs.next())
 			{
 				try
 				{
 					String javaClassName = rs.getString(1);
 					String sqlName = rs.getString(2);
-					Class cls = loader.loadClass(javaClassName);
+					Class<?> cls = loader.loadClass(javaClassName);
 					if(!SQLData.class.isAssignableFrom(cls))
-						throw new SQLException("Class " + javaClassName + " does not implement java.sql.SQLData");
+						throw new SQLException("Class " + javaClassName +
+							" does not implement java.sql.SQLData");
 					
 					Oid typeOid = Oid.forTypeName(sqlName);
 					typesForSchema.put(typeOid, cls.asSubclass(SQLData.class));
-					s_logger.finer("Adding type mapping for OID " + typeOid + " -> class " + cls.getName() + " for schema " + schema);
+					s_logger.finer("Adding type mapping for OID " + typeOid +
+						" -> class " + cls.getName() + " for schema " + schema);
 				}
 				catch(ClassNotFoundException e)
 				{
@@ -296,55 +343,87 @@ public class Loader extends ClassLoader
 				}
 			}
 			if(typesForSchema.isEmpty())
-				typesForSchema = Collections.EMPTY_MAP;
+				typesForSchema = Map.of();
 			s_typeMap.put(schema, typesForSchema);
 			return typesForSchema;
 		}
-		finally
-		{
-			SQLUtils.close(rs);
-			SQLUtils.close(stmt);
-		}
 	}
 
 	private static URL entryURL(int entryId)
 	{
 		try
 		{
-			return new URL(
+			return doPrivileged(() -> new URL(
 					"dbf",
 					"localhost",
 					-1,
 					"/" + entryId,
-					EntryStreamHandler.getInstance());
+					EntryStreamHandler.getInstance()));
 		}
 		catch(MalformedURLException e)
 		{
-			throw new RuntimeException(e);
+			throw unchecked(e);
 		}
 	}
 
-	private final Map m_entries;
+	/**
+	 * Map from name of entry (resource or expanded class name) to an array of
+	 * the integer surrogate keys for jar entries, in the order of jars on this
+	 * loader's jar path that contain entries matching the name.
+	 */
+	private final Map<String,int[]> m_entries;
+	private final Map<Integer,ProtectionDomain> m_domains;
+
+	/**
+	 * Private constructor used only to create the "sentinel" (non-)loader.
+	 *<p>
+	 * Any attempt to use it will incur null pointer exceptions, but it would be
+	 * a bug already for such use to be attempted.
+	 */
+	private Loader()
+	{
+		m_entries  = null;
+		m_domains  = null;
+		m_j9Helper = null;
+	}
 
 	/**
 	 * Create a new Loader.
 	 * @param entries
 	 * @param parent
 	 */
-	Loader(Map entries, ClassLoader parent)
+	Loader(
+		Map<String,int[]> entries,
+		Map<Integer,CodeSource> sources, ClassLoader parent, String name)
 	{
-		super(parent);
+		super(name, parent);
 		m_entries = entries;
 		m_j9Helper = ifJ9getHelper(); // null if not under OpenJ9 with sharing
+
+		Principal[] noPrincipals = new Principal[0];
+
+		m_domains = new HashMap<>();
+
+		sources.entrySet().stream()
+			.collect(groupingBy(Map.Entry::getValue))
+			.entrySet().stream().forEach(e ->
+			{
+				ProtectionDomain pd = new ProtectionDomain(
+					e.getKey(), null /* no permissions */, this, noPrincipals);
+				e.getValue().forEach(ee -> m_domains.put(ee.getKey(), pd));
+			});
 	}
 
+	@Override
 	protected Class<?> findClass(final String name)
 	throws ClassNotFoundException
 	{
 		String path = name.replace('.', '/').concat(".class");
-		int[] entryId = (int[])m_entries.get(path);
+		int[] entryId = m_entries.get(path);
 		if(entryId != null)
 		{
+			ProtectionDomain pd = m_domains.get(entryId[0]);
+
 			/*
 			 * Check early whether running on OpenJ9 JVM and the shared cache
 			 * has the class. It is possible this early because the entryId is
@@ -359,32 +438,38 @@ public class Loader extends ClassLoader
 			if ( o instanceof byte[] )
 			{
 				byte[] img = (byte[]) o;
-				return defineClass(name, img, 0, img.length);
+				return defineClass(name, img, 0, img.length, pd);
 			}
 			String ifJ9token = (String) o; // used below when storing class
 
-			PreparedStatement stmt = null;
-			ResultSet rs = null;
-			try
-			{
+			try (
 				// This code relies heavily on the fact that the connection
 				// is a singleton and that the prepared statement will live
-				// for the duration of the loader.
+				// for the duration of the loader. (This comment has said so
+				// since January 2004; the prepared statement has been getting
+				// closed in a finally block since November 2004, and that
+				// hasn't broken anything, and it is currently true that
+				// prepared statements are backed by ExecutionPlans that stick
+				// around in an MRU cache after being closed.)
 				//
-				stmt = SQLUtils.getDefaultConnection().prepareStatement(
-					"SELECT entryImage FROM sqlj.jar_entry " +
-					"WHERE entryId OPERATOR(pg_catalog.=) ?");
-
-				stmt.setInt(1, entryId[0]);
-				stmt.unwrap(SPIReadOnlyControl.class).clearReadOnly();
-				rs = stmt.executeQuery();
+				PreparedStatement stmt = Checked.Supplier.use((() ->
+					{
+						PreparedStatement s = getDefaultConnection()
+							.prepareStatement(
+								"SELECT entryImage FROM sqlj.jar_entry " +
+								"WHERE entryId OPERATOR(pg_catalog.=) ?");
+						s.unwrap(SPIReadOnlyControl.class).clearReadOnly();
+						s.setInt(1, entryId[0]);
+						return s;
+					})).get();
+				ResultSet rs = stmt.executeQuery();
+			)
+			{
 				if(rs.next())
 				{
 					byte[] img = rs.getBytes(1);
-					rs.close();
-					rs = null;
 
-					Class<?> cls = this.defineClass(name, img, 0, img.length);
+					Class<?> cls = defineClass(name, img, 0, img.length, pd);
 
 					ifJ9storeSharedClass(ifJ9token, cls); // noop for null token
 					return cls;
@@ -392,31 +477,30 @@ public class Loader extends ClassLoader
 			}
 			catch(SQLException e)
 			{
-				Logger.getAnonymousLogger().log(Level.INFO, "Failed to load class", e);
-				throw new ClassNotFoundException(name + " due to: " + e.getMessage());
-			}
-			finally
-			{
-				SQLUtils.close(rs);
-				SQLUtils.close(stmt);
+				Logger.getAnonymousLogger().log(Level.INFO,
+					"Failed to load class", e);
+				throw new ClassNotFoundException(name + " due to: " +
+					e.getMessage(), e);
 			}
 		}
-	throw new ClassNotFoundException(name);
+		throw new ClassNotFoundException(name);
 	}
 
+	@Override
 	protected URL findResource(String name)
 	{
-		int[] entryIds = (int[])m_entries.get(name);
+		int[] entryIds = m_entries.get(name);
 		if(entryIds == null)
 			return null;
 		
 		return entryURL(entryIds[0]);
 	}
 
-	protected Enumeration findResources(String name)
+	@Override
+	protected Enumeration<URL> findResources(String name)
     throws IOException
 	{
-		int[] entryIds = (int[])m_entries.get(name);
+		int[] entryIds = m_entries.get(name);
 		if(entryIds == null)
 			entryIds = new int[0];
 		return new EntryEnumeration(entryIds);
@@ -429,9 +513,9 @@ public class Loader extends ClassLoader
 	 */
 
 	private static final Object s_j9HelperFactory;
-	private static final Method s_j9GetTokenHelper;
-	private static final Method s_j9FindSharedClass;
-	private static final Method s_j9StoreSharedClass;
+	private static final MethodHandle s_j9GetTokenHelper;
+	private static final MethodHandle s_j9FindSharedClass;
+	private static final MethodHandle s_j9StoreSharedClass;
 	private final Object m_j9Helper;
 
 	/**
@@ -446,18 +530,9 @@ public class Loader extends ClassLoader
 		{
 			return s_j9GetTokenHelper.invoke(s_j9HelperFactory, this);
 		}
-		catch ( IllegalAccessException e )
+		catch ( Throwable t )
 		{
-			throw new SecurityException(e);
-		}
-		catch ( InvocationTargetException ite )
-		{
-			Throwable t = ite.getCause();
-			if ( t instanceof Error )
-				throw (Error)t;
-			if ( t instanceof RuntimeException )
-				throw (RuntimeException)t;
-			throw new UndeclaredThrowableException(t, t.getMessage());
+			throw unchecked(t);
 		}
 	}
 
@@ -490,18 +565,9 @@ public class Loader extends ClassLoader
 				return token;
 			return cookie;
 		}
-		catch ( IllegalAccessException e )
+		catch ( Throwable t )
 		{
-			throw new SecurityException(e);
-		}
-		catch ( InvocationTargetException ite )
-		{
-			Throwable t = ite.getCause();
-			if ( t instanceof Error )
-				throw (Error)t;
-			if ( t instanceof RuntimeException )
-				throw (RuntimeException)t;
-			throw new UndeclaredThrowableException(t, t.getMessage());
+			throw unchecked(t);
 		}
 	}
 
@@ -524,18 +590,9 @@ public class Loader extends ClassLoader
 		{
 			s_j9StoreSharedClass.invoke(m_j9Helper, token, cls);
 		}
-		catch ( IllegalAccessException e )
+		catch ( Throwable t )
 		{
-			throw new SecurityException(e);
-		}
-		catch ( InvocationTargetException ite )
-		{
-			Throwable t = ite.getCause();
-			if ( t instanceof Error )
-				throw (Error)t;
-			if ( t instanceof RuntimeException )
-				throw (RuntimeException)t;
-			throw new UndeclaredThrowableException(t, t.getMessage());
+			throw unchecked(t);
 		}
 	}
 
@@ -546,9 +603,9 @@ public class Loader extends ClassLoader
 	static
 	{
 		Object factory = null;
-		Method getHelper = null;
-		Method findShared = null;
-		Method storeShared = null;
+		MethodHandle getHelper = null;
+		MethodHandle findShared = null;
+		MethodHandle storeShared = null;
 
 		try
 		{
@@ -556,36 +613,36 @@ public class Loader extends ClassLoader
 			Class<?> shared = ClassLoader.getSystemClassLoader().loadClass(
 				"com.ibm.oti.shared.Shared");
 
-			Method getFactory =
-				shared.getMethod("getSharedClassHelperFactory", null);
+			MethodHandles.Lookup lup = MethodHandles.publicLookup();
+
+			MethodHandle getFactory = lup.unreflect(shared.getMethod(
+				"getSharedClassHelperFactory", (Class<?>[])null));
 
 			/* If getFactory returns null, sharing is not enabled. */
-			factory = getFactory.invoke(null);
+			factory = getFactory.invoke();
 			if ( null != factory )
 			{
-				Class<?> factoryClass = getFactory.getReturnType();
-				getHelper =
-					factoryClass.getMethod("getTokenHelper", ClassLoader.class);
-				Class<?> helperClass = getHelper.getReturnType();
-				findShared =
-					helperClass.getMethod(
-						"findSharedClass", String.class, String.class);
-				storeShared =
-					helperClass.getMethod(
-						"storeSharedClass", String.class, Class.class);
+				Class<?> factoryClass = getFactory.type().returnType();
+				getHelper = lup.unreflect(
+					factoryClass.getMethod("getTokenHelper",ClassLoader.class));
+				Class<?> helperClass = getHelper.type().returnType();
+				findShared = lup.findVirtual(helperClass, "findSharedClass",
+					methodType(byte[].class, String.class, String.class));
+				storeShared = lup.findVirtual(helperClass, "storeSharedClass",
+					methodType(boolean.class, String.class, Class.class));
 			}
 		}
 		catch ( ClassNotFoundException cnfe )
 		{
 			/* Not running on an OpenJ9 JVM. Leave all the statics null. */
 		}
-		catch ( RuntimeException rte )
+		catch ( Error | RuntimeException e )
 		{
-			throw rte;
+			throw e;
 		}
-		catch ( Exception e )
+		catch ( Throwable t )
 		{
-			throw new ExceptionInInitializerError(e);
+			throw new ExceptionInInitializerError(t);
 		}
 		finally
 		{
